@@ -7,7 +7,7 @@
  * Platform-specific module methods for FreeBSD and OpenBSD.
 
  * OpenBSD references:
- * - OpenBSD source code: http://anoncvs.spacehopper.org/openbsd-src/
+ * - OpenBSD source code: https://github.com/openbsd/src
  *
  * OpenBSD / NetBSD: missing APIs compared to FreeBSD implementation:
  * - psutil.net_connections()
@@ -30,7 +30,9 @@
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/sysctl.h>
+#if !defined(__NetBSD__)
 #include <sys/user.h>
+#endif
 #include <sys/proc.h>
 #include <sys/file.h>
 #include <sys/socket.h>
@@ -60,10 +62,12 @@
 #include <sys/un.h>
 
 #include "_psutil_common.h"
+#include "_psutil_posix.h"
 
 #ifdef PSUTIL_FREEBSD
-    #include "arch/bsd/freebsd.h"
-    #include "arch/bsd/freebsd_socks.h"
+    #include "arch/freebsd/specific.h"
+    #include "arch/freebsd/sys_socks.h"
+    #include "arch/freebsd/proc_socks.h"
 
     #include <net/if_media.h>
     #include <devstat.h>  // get io counters
@@ -74,7 +78,7 @@
         #include <utmpx.h>
     #endif
 #elif PSUTIL_OPENBSD
-    #include "arch/bsd/openbsd.h"
+    #include "arch/openbsd/specific.h"
 
     #include <utmp.h>
     #include <sys/vnode.h>  // for VREG
@@ -83,8 +87,8 @@
     #undef _KERNEL
     #include <sys/sched.h>  // for CPUSTATES & CP_*
 #elif PSUTIL_NETBSD
-    #include "arch/bsd/netbsd.h"
-    #include "arch/bsd/netbsd_socks.h"
+    #include "arch/netbsd/specific.h"
+    #include "arch/netbsd/socks.h"
 
     #include <utmpx.h>
     #include <sys/vnode.h>  // for VREG
@@ -178,10 +182,8 @@ psutil_boot_time(PyObject *self, PyObject *args) {
     struct timeval boottime;
     size_t len = sizeof(boottime);
 
-    if (sysctl(request, 2, &boottime, &len, NULL, 0) == -1) {
-        PyErr_SetFromErrno(PyExc_OSError);
-        return NULL;
-    }
+    if (sysctl(request, 2, &boottime, &len, NULL, 0) == -1)
+        return PyErr_SetFromErrno(PyExc_OSError);
     return Py_BuildValue("d", (double)boottime.tv_sec);
 }
 
@@ -198,7 +200,7 @@ psutil_proc_oneshot_info(PyObject *self, PyObject *args) {
     long memtext;
     long memdata;
     long memstack;
-    unsigned char oncpu;
+    int oncpu;
     kinfo_proc kp;
     long pagesize = sysconf(_SC_PAGESIZE);
     char str[1000];
@@ -216,17 +218,14 @@ psutil_proc_oneshot_info(PyObject *self, PyObject *args) {
 #elif defined(PSUTIL_OPENBSD) || defined(PSUTIL_NETBSD)
     sprintf(str, "%s", kp.p_comm);
 #endif
-#if PY_MAJOR_VERSION >= 3
     py_name = PyUnicode_DecodeFSDefault(str);
-#else
-    py_name = Py_BuildValue("s", str);
-#endif
     if (! py_name) {
         // Likely a decoding error. We don't want to fail the whole
         // operation. The python module may retry with proc_name().
         PyErr_Clear();
         py_name = Py_None;
     }
+    // Py_INCREF(py_name);
 
     // Calculate memory.
 #ifdef PSUTIL_FREEBSD
@@ -239,13 +238,13 @@ psutil_proc_oneshot_info(PyObject *self, PyObject *args) {
     rss = (long)kp.p_vm_rssize * pagesize;
     #ifdef PSUTIL_OPENBSD
         // VMS, this is how ps determines it on OpenBSD:
-        // http://anoncvs.spacehopper.org/openbsd-src/tree/bin/ps/print.c#n461
-        // vms
+        // https://github.com/openbsd/src/blob/
+        //     588f7f8c69786211f2d16865c552afb91b1c7cba/bin/ps/print.c#L505
         vms = (long)(kp.p_vm_dsize + kp.p_vm_ssize + kp.p_vm_tsize) * pagesize;
     #elif PSUTIL_NETBSD
         // VMS, this is how top determines it on NetBSD:
-        // ftp://ftp.iij.ad.jp/pub/NetBSD/NetBSD-release-6/src/external/bsd/
-        //     top/dist/machine/m_netbsd.c
+        // https://github.com/IIJ-NetBSD/netbsd-src/blob/master/external/
+        //     bsd/top/dist/machine/m_netbsd.c
         vms = (long)kp.p_vm_msize * pagesize;
     #endif
         memtext = (long)kp.p_vm_tsize * pagesize;
@@ -257,6 +256,7 @@ psutil_proc_oneshot_info(PyObject *self, PyObject *args) {
     // what CPU we're on; top was used as an example:
     // https://svnweb.freebsd.org/base/head/usr.bin/top/machine.c?
     //     view=markup&pathrev=273835
+    // XXX - note: for "intr" PID this is -1.
     if (kp.ki_stat == SRUN && kp.ki_oncpu != NOCPU)
         oncpu = kp.ki_oncpu;
     else
@@ -305,7 +305,7 @@ psutil_proc_oneshot_info(PyObject *self, PyObject *args) {
         memdata,                          // (long) mem data
         memstack,                         // (long) mem stack
         // others
-        oncpu,                            // (unsigned char) the CPU we are on
+        oncpu,                            // (int) the CPU we are on
 #elif defined(PSUTIL_OPENBSD) || defined(PSUTIL_NETBSD)
         //
         (long)kp.p_ppid,                 // (long) ppid
@@ -341,15 +341,12 @@ psutil_proc_oneshot_info(PyObject *self, PyObject *args) {
         memdata,                          // (long) mem data
         memstack,                         // (long) mem stack
         // others
-        oncpu,                            // (unsigned char) the CPU we are on
+        oncpu,                            // (int) the CPU we are on
 #endif
         py_name                           // (pystr) name
     );
 
-    if (py_retlist != NULL) {
-        // XXX shall we decref() also in case of Py_BuildValue() error?
-        Py_DECREF(py_name);
-    }
+    Py_DECREF(py_name);
     return py_retlist;
 }
 
@@ -373,12 +370,7 @@ psutil_proc_name(PyObject *self, PyObject *args) {
 #elif defined(PSUTIL_OPENBSD) || defined(PSUTIL_NETBSD)
     sprintf(str, "%s", kp.p_comm);
 #endif
-
-#if PY_MAJOR_VERSION >= 3
     return PyUnicode_DecodeFSDefault(str);
-#else
-    return Py_BuildValue("s", str);
-#endif
 }
 
 
@@ -392,19 +384,16 @@ psutil_proc_cmdline(PyObject *self, PyObject *args) {
 
     if (! PyArg_ParseTuple(args, "l", &pid))
         return NULL;
-
     py_retlist = psutil_get_cmdline(pid);
-    // psutil_get_cmdline() returns NULL only if psutil_cmd_args
-    // failed with ESRCH (no process with that PID)
-    if (NULL == py_retlist)
-        return PyErr_SetFromErrno(PyExc_OSError);
+    if (py_retlist == NULL)
+        return NULL;
     return Py_BuildValue("N", py_retlist);
 }
 
 
 /*
  * Return the number of logical CPUs in the system.
- * XXX this could be shared with OSX
+ * XXX this could be shared with macOS
  */
 static PyObject *
 psutil_cpu_count_logical(PyObject *self, PyObject *args) {
@@ -442,11 +431,8 @@ psutil_cpu_times(PyObject *self, PyObject *args) {
     int mib[] = {CTL_KERN, KERN_CPTIME};
     ret = sysctl(mib, 2, &cpu_time, &size, NULL, 0);
 #endif
-    if (ret == -1) {
-        PyErr_SetFromErrno(PyExc_OSError);
-        return NULL;
-    }
-
+    if (ret == -1)
+        return PyErr_SetFromErrno(PyExc_OSError);
     return Py_BuildValue("(ddddd)",
                          (double)cpu_time[CP_USER] / CLOCKS_PER_SEC,
                          (double)cpu_time[CP_NICE] / CLOCKS_PER_SEC,
@@ -467,12 +453,17 @@ psutil_cpu_times(PyObject *self, PyObject *args) {
 static PyObject *
 psutil_proc_open_files(PyObject *self, PyObject *args) {
     long pid;
-    int i, cnt;
+    int i;
+    int cnt;
+    int regular;
+    int fd;
+    char *path;
     struct kinfo_file *freep = NULL;
     struct kinfo_file *kif;
     kinfo_proc kipp;
-    PyObject *py_retlist = PyList_New(0);
     PyObject *py_tuple = NULL;
+    PyObject *py_path = NULL;
+    PyObject *py_retlist = PyList_New(0);
 
     if (py_retlist == NULL)
         return NULL;
@@ -484,32 +475,39 @@ psutil_proc_open_files(PyObject *self, PyObject *args) {
     errno = 0;
     freep = kinfo_getfile(pid, &cnt);
     if (freep == NULL) {
-        psutil_raise_for_pid(pid, "kinfo_getfile() failed");
+        psutil_raise_for_pid(pid, "kinfo_getfile()");
         goto error;
     }
 
     for (i = 0; i < cnt; i++) {
         kif = &freep[i];
+
 #ifdef PSUTIL_FREEBSD
-        if ((kif->kf_type == KF_TYPE_VNODE) &&
-                (kif->kf_vnode_type == KF_VTYPE_VREG))
-        {
-            py_tuple = Py_BuildValue("(si)", kif->kf_path, kif->kf_fd);
+        regular = (kif->kf_type == KF_TYPE_VNODE) && \
+            (kif->kf_vnode_type == KF_VTYPE_VREG);
+        fd = kif->kf_fd;
+        path = kif->kf_path;
 #elif PSUTIL_OPENBSD
-        if ((kif->f_type == DTYPE_VNODE) &&
-                (kif->v_type == VREG))
-        {
-            py_tuple = Py_BuildValue("(si)", "", kif->fd_fd);
+        regular = (kif->f_type == DTYPE_VNODE) && (kif->v_type == VREG);
+        fd = kif->fd_fd;
+        // XXX - it appears path is not exposed in the kinfo_file struct.
+        path = "";
 #elif PSUTIL_NETBSD
-        if ((kif->ki_ftype == DTYPE_VNODE) &&
-                (kif->ki_vtype == VREG))
-        {
-            py_tuple = Py_BuildValue("(si)", "", kif->ki_fd);
+        regular = (kif->ki_ftype == DTYPE_VNODE) && (kif->ki_vtype == VREG);
+        fd = kif->ki_fd;
+        // XXX - it appears path is not exposed in the kinfo_file struct.
+        path = "";
 #endif
+        if (regular == 1) {
+            py_path = PyUnicode_DecodeFSDefault(path);
+            if (! py_path)
+                goto error;
+            py_tuple = Py_BuildValue("(Oi)", py_path, fd);
             if (py_tuple == NULL)
                 goto error;
             if (PyList_Append(py_retlist, py_tuple))
                 goto error;
+            Py_DECREF(py_path);
             Py_DECREF(py_tuple);
         }
     }
@@ -543,6 +541,8 @@ psutil_disk_partitions(PyObject *self, PyObject *args) {
     struct statfs *fs = NULL;
 #endif
     PyObject *py_retlist = PyList_New(0);
+    PyObject *py_dev = NULL;
+    PyObject *py_mountp = NULL;
     PyObject *py_tuple = NULL;
 
     if (py_retlist == NULL)
@@ -655,15 +655,23 @@ psutil_disk_partitions(PyObject *self, PyObject *args) {
         if (flags & MNT_NODEVMTIME)
             strlcat(opts, ",nodevmtime", sizeof(opts));
 #endif
-        py_tuple = Py_BuildValue("(ssss)",
-                                 fs[i].f_mntfromname,  // device
-                                 fs[i].f_mntonname,    // mount point
+        py_dev = PyUnicode_DecodeFSDefault(fs[i].f_mntfromname);
+        if (! py_dev)
+            goto error;
+        py_mountp = PyUnicode_DecodeFSDefault(fs[i].f_mntonname);
+        if (! py_mountp)
+            goto error;
+        py_tuple = Py_BuildValue("(OOss)",
+                                 py_dev,               // device
+                                 py_mountp,            // mount point
                                  fs[i].f_fstypename,   // fs type
                                  opts);                // options
         if (!py_tuple)
             goto error;
         if (PyList_Append(py_retlist, py_tuple))
             goto error;
+        Py_DECREF(py_dev);
+        Py_DECREF(py_mountp);
         Py_DECREF(py_tuple);
     }
 
@@ -671,6 +679,8 @@ psutil_disk_partitions(PyObject *self, PyObject *args) {
     return py_retlist;
 
 error:
+    Py_XDECREF(py_dev);
+    Py_XDECREF(py_mountp);
     Py_XDECREF(py_tuple);
     Py_DECREF(py_retlist);
     if (fs != NULL)
@@ -780,6 +790,9 @@ error:
 static PyObject *
 psutil_users(PyObject *self, PyObject *args) {
     PyObject *py_retlist = PyList_New(0);
+    PyObject *py_username = NULL;
+    PyObject *py_tty = NULL;
+    PyObject *py_hostname = NULL;
     PyObject *py_tuple = NULL;
 
     if (py_retlist == NULL)
@@ -791,19 +804,34 @@ psutil_users(PyObject *self, PyObject *args) {
 
     fp = fopen(_PATH_UTMP, "r");
     if (fp == NULL) {
-        PyErr_SetFromErrno(PyExc_OSError);
+        PyErr_SetFromErrnoWithFilename(PyExc_OSError, _PATH_UTMP);
         goto error;
     }
 
     while (fread(&ut, sizeof(ut), 1, fp) == 1) {
         if (*ut.ut_name == '\0')
             continue;
+        py_username = PyUnicode_DecodeFSDefault(ut.ut_name);
+        if (! py_username)
+            goto error;
+        py_tty = PyUnicode_DecodeFSDefault(ut.ut_line);
+        if (! py_tty)
+            goto error;
+        py_hostname = PyUnicode_DecodeFSDefault(ut.ut_host);
+        if (! py_hostname)
+            goto error;
         py_tuple = Py_BuildValue(
-            "(sssf)",
-            ut.ut_name,         // username
-            ut.ut_line,         // tty
-            ut.ut_host,         // hostname
-           (float)ut.ut_time);  // start time
+            "(OOOfi)",
+            py_username,        // username
+            py_tty,             // tty
+            py_hostname,        // hostname
+            (float)ut.ut_time,  // start time
+#ifdef PSUTIL_OPENBSD
+            -1                  // process id (set to None later)
+#else
+            ut.ut_pid           // process id
+#endif
+        );
         if (!py_tuple) {
             fclose(fp);
             goto error;
@@ -812,23 +840,39 @@ psutil_users(PyObject *self, PyObject *args) {
             fclose(fp);
             goto error;
         }
+        Py_DECREF(py_username);
+        Py_DECREF(py_tty);
+        Py_DECREF(py_hostname);
         Py_DECREF(py_tuple);
     }
 
     fclose(fp);
 #else
     struct utmpx *utx;
-
     setutxent();
     while ((utx = getutxent()) != NULL) {
         if (utx->ut_type != USER_PROCESS)
             continue;
+        py_username = PyUnicode_DecodeFSDefault(utx->ut_user);
+        if (! py_username)
+            goto error;
+        py_tty = PyUnicode_DecodeFSDefault(utx->ut_line);
+        if (! py_tty)
+            goto error;
+        py_hostname = PyUnicode_DecodeFSDefault(utx->ut_host);
+        if (! py_hostname)
+            goto error;
         py_tuple = Py_BuildValue(
-            "(sssf)",
-            utx->ut_user,  // username
-            utx->ut_line,  // tty
-            utx->ut_host,  // hostname
-            (float)utx->ut_tv.tv_sec  // start time
+            "(OOOfi)",
+            py_username,   // username
+            py_tty,        // tty
+            py_hostname,   // hostname
+            (float)utx->ut_tv.tv_sec,  // start time
+#ifdef PSUTIL_OPENBSD
+            -1             // process id (set to None later)
+#else
+            utx->ut_pid    // process id
+#endif
         );
 
         if (!py_tuple) {
@@ -839,6 +883,9 @@ psutil_users(PyObject *self, PyObject *args) {
             endutxent();
             goto error;
         }
+        Py_DECREF(py_username);
+        Py_DECREF(py_tty);
+        Py_DECREF(py_hostname);
         Py_DECREF(py_tuple);
     }
 
@@ -847,6 +894,9 @@ psutil_users(PyObject *self, PyObject *args) {
     return py_retlist;
 
 error:
+    Py_XDECREF(py_username);
+    Py_XDECREF(py_tty);
+    Py_XDECREF(py_hostname);
     Py_XDECREF(py_tuple);
     Py_DECREF(py_retlist);
     return NULL;
@@ -858,40 +908,35 @@ error:
  */
 static PyMethodDef
 PsutilMethods[] = {
-
     // --- per-process functions
 
     {"proc_oneshot_info", psutil_proc_oneshot_info, METH_VARARGS,
      "Return multiple info about a process"},
     {"proc_name", psutil_proc_name, METH_VARARGS,
      "Return process name"},
-#if !defined(PSUTIL_NETBSD)
-    {"proc_connections", psutil_proc_connections, METH_VARARGS,
-     "Return connections opened by process"},
-#endif
     {"proc_cmdline", psutil_proc_cmdline, METH_VARARGS,
      "Return process cmdline as a list of cmdline arguments"},
     {"proc_threads", psutil_proc_threads, METH_VARARGS,
      "Return process threads"},
 #if defined(PSUTIL_FREEBSD) || defined(PSUTIL_OPENBSD)
+    {"proc_connections", psutil_proc_connections, METH_VARARGS,
+     "Return connections opened by process"},
     {"proc_cwd", psutil_proc_cwd, METH_VARARGS,
      "Return process current working directory."},
 #endif
 #if defined(__FreeBSD_version) && __FreeBSD_version >= 800000 || PSUTIL_OPENBSD || defined(PSUTIL_NETBSD)
     {"proc_num_fds", psutil_proc_num_fds, METH_VARARGS,
      "Return the number of file descriptors opened by this process"},
-#endif
-#if defined(__FreeBSD_version) && __FreeBSD_version >= 800000 || PSUTIL_OPENBSD || defined(PSUTIL_NETBSD)
     {"proc_open_files", psutil_proc_open_files, METH_VARARGS,
      "Return files opened by process as a list of (path, fd) tuples"},
 #endif
-
 #if defined(PSUTIL_FREEBSD) || defined(PSUTIL_NETBSD)
-    {"proc_exe", psutil_proc_exe, METH_VARARGS,
-     "Return process pathname executable"},
     {"proc_num_threads", psutil_proc_num_threads, METH_VARARGS,
      "Return number of threads used by process"},
+#endif
 #if defined(PSUTIL_FREEBSD)
+    {"proc_exe", psutil_proc_exe, METH_VARARGS,
+     "Return process pathname executable"},
     {"proc_memory_maps", psutil_proc_memory_maps, METH_VARARGS,
      "Return a list of tuples for every process's memory map"},
     {"proc_cpu_affinity_get", psutil_proc_cpu_affinity_get, METH_VARARGS,
@@ -900,7 +945,6 @@ PsutilMethods[] = {
      "Set process CPU affinity."},
     {"cpu_count_phys", psutil_cpu_count_phys, METH_VARARGS,
      "Return an XML string to determine the number physical CPUs."},
-#endif
 #endif
 
     // --- system-related functions
@@ -937,7 +981,16 @@ PsutilMethods[] = {
 #if defined(PSUTIL_FREEBSD)
     {"sensors_battery", psutil_sensors_battery, METH_VARARGS,
      "Return battery information."},
+    {"sensors_cpu_temperature", psutil_sensors_cpu_temperature, METH_VARARGS,
+     "Return temperature information for a given CPU core number."},
+    {"cpu_frequency", psutil_cpu_freq, METH_VARARGS,
+     "Return frequency of a given CPU"},
 #endif
+
+    // --- others
+    {"set_testing", psutil_set_testing, METH_NOARGS,
+     "Set psutil in testing mode"},
+
     {NULL, NULL, 0, NULL}
 };
 
@@ -1039,6 +1092,8 @@ void init_psutil_bsd(void)
     // PSUTIL_CONN_NONE
     PyModule_AddIntConstant(module, "PSUTIL_CONN_NONE", 128);
 
+    psutil_setup();
+
     if (module == NULL)
         INITERROR;
 #if PY_MAJOR_VERSION >= 3
