@@ -9,18 +9,23 @@
  * this in Cython which I later on translated in C.
  */
 
+/* fix compilation issue on SunOS 5.10, see:
+ * https://github.com/giampaolo/psutil/issues/421
+ * https://github.com/giampaolo/psutil/issues/1077
+ * http://us-east.manta.joyent.com/jmc/public/opensolaris/ARChive/PSARC/2010/111/materials/s10ceval.txt
+ *
+ * Because LEGACY_MIB_SIZE defined in the same file there is no way to make autoconfiguration =\
+*/
+
+#define NEW_MIB_COMPLIANT 1
+#define _STRUCTURED_PROC 1
 
 #include <Python.h>
 
-// fix for "Cannot use procfs in the large file compilation environment"
-// error, see:
-// http://sourceware.org/ml/gdb-patches/2010-11/msg00336.html
-#undef _FILE_OFFSET_BITS
-#define _STRUCTURED_PROC 1
-
-// fix compilation issue on SunOS 5.10, see:
-// https://github.com/giampaolo/psutil/issues/421
-#define NEW_MIB_COMPLIANT
+#if !defined(_LP64) && _FILE_OFFSET_BITS == 64
+#  undef _FILE_OFFSET_BITS
+#  undef _LARGEFILE64_SOURCE
+#endif
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -41,23 +46,23 @@
 #include <inet/tcp.h>
 #include <arpa/inet.h>
 #include <net/if.h>
+#include <math.h> // fabs()
 
+#include "_psutil_common.h"
+#include "_psutil_posix.h"
+
+#include "arch/solaris/environ.h"
 
 #define PSUTIL_TV2DOUBLE(t) (((t).tv_nsec * 0.000000001) + (t).tv_sec)
-#ifndef EXPER_IP_AND_ALL_IRES
-#define EXPER_IP_AND_ALL_IRES (1024+4)
-#endif
-// a signaler for connections without an actual status
-static int PSUTIL_CONN_NONE = 128;
 
 
 /*
  * Read a file content and fills a C structure with it.
  */
-int
+static int
 psutil_file_to_struct(char *path, void *fstruct, size_t size) {
     int fd;
-    size_t nbytes;
+    ssize_t nbytes;
     fd = open(path, O_RDONLY);
     if (fd == -1) {
         PyErr_SetFromErrnoWithFilename(PyExc_OSError, path);
@@ -69,7 +74,7 @@ psutil_file_to_struct(char *path, void *fstruct, size_t size) {
         PyErr_SetFromErrno(PyExc_OSError);
         return 0;
     }
-    if (nbytes != size) {
+    if (nbytes != (ssize_t) size) {
         close(fd);
         PyErr_SetString(
             PyExc_RuntimeError, "read() file structure size mismatch");
@@ -97,18 +102,75 @@ psutil_proc_basic_info(PyObject *self, PyObject *args) {
     sprintf(path, "%s/%i/psinfo", procfs_path, pid);
     if (! psutil_file_to_struct(path, (void *)&info, sizeof(info)))
         return NULL;
-    return Py_BuildValue("ikkdiiik",
-                         info.pr_ppid,              // parent pid
-                         info.pr_rssize,            // rss
-                         info.pr_size,              // vms
-                         PSUTIL_TV2DOUBLE(info.pr_start),  // create time
-                         info.pr_lwp.pr_nice,       // nice
-                         info.pr_nlwp,              // no. of threads
-                         info.pr_lwp.pr_state,      // status code
-                         info.pr_ttydev             // tty nr
-                        );
+    return Py_BuildValue(
+        "ikkdiiikiiii",
+        info.pr_ppid,              // parent pid
+        info.pr_rssize,            // rss
+        info.pr_size,              // vms
+        PSUTIL_TV2DOUBLE(info.pr_start),  // create time
+        info.pr_lwp.pr_nice,       // nice
+        info.pr_nlwp,              // no. of threads
+        info.pr_lwp.pr_state,      // status code
+        info.pr_ttydev,            // tty nr
+        (int)info.pr_uid,          // real user id
+        (int)info.pr_euid,         // effective user id
+        (int)info.pr_gid,          // real group id
+        (int)info.pr_egid          // effective group id
+        );
 }
 
+/*
+ * Join array of C strings to C string with delemiter dm.
+ * Omit empty records.
+ */
+static int
+cstrings_array_to_string(char **joined, char ** array, size_t count, char dm) {
+    int i;
+    size_t total_length = 0;
+    size_t item_length = 0;
+    char *result = NULL;
+    char *last = NULL;
+
+    if (!array || !joined)
+        return 0;
+
+    for (i=0; i<count; i++) {
+        if (!array[i])
+            continue;
+
+        item_length = strlen(array[i]) + 1;
+        total_length += item_length;
+    }
+
+    if (!total_length) {
+        return 0;
+    }
+
+    result = malloc(total_length);
+    if (!result) {
+        PyErr_NoMemory();
+        return -1;
+    }
+
+    result[0] = '\0';
+    last = result;
+
+    for (i=0; i<count; i++) {
+        if (!array[i])
+            continue;
+
+        item_length = strlen(array[i]);
+        memcpy(last, array[i], item_length);
+        last[item_length] = dm;
+
+        last += item_length + 1;
+    }
+
+    result[total_length-1] = '\0';
+    *joined = result;
+
+    return total_length;
+}
 
 /*
  * Return process name and args as a Python tuple.
@@ -118,9 +180,14 @@ psutil_proc_name_and_args(PyObject *self, PyObject *args) {
     int pid;
     char path[1000];
     psinfo_t info;
+    size_t argc;
+    int joined;
+    char **argv;
+    char *argv_plain;
     const char *procfs_path;
-    PyObject *py_name;
-    PyObject *py_args;
+    PyObject *py_name = NULL;
+    PyObject *py_args = NULL;
+    PyObject *py_retlist = NULL;
 
     if (! PyArg_ParseTuple(args, "is", &pid, &procfs_path))
         return NULL;
@@ -128,17 +195,126 @@ psutil_proc_name_and_args(PyObject *self, PyObject *args) {
     if (! psutil_file_to_struct(path, (void *)&info, sizeof(info)))
         return NULL;
 
-#if PY_MAJOR_VERSION >= 3
     py_name = PyUnicode_DecodeFSDefault(info.pr_fname);
     if (!py_name)
-        return NULL;
-    py_args = PyUnicode_DecodeFSDefault(info.pr_psargs);
+        goto error;
+
+    /* SunOS truncates arguments to length PRARGSZ, very likely args are truncated.
+     * The only way to retrieve full command line is to parse process memory */
+    if (info.pr_argc && strlen(info.pr_psargs) == PRARGSZ-1) {
+        argv = psutil_read_raw_args(info, procfs_path, &argc);
+        if (argv) {
+            joined = cstrings_array_to_string(&argv_plain, argv, argc, ' ');
+            if (joined > 0) {
+                py_args = PyUnicode_DecodeFSDefault(argv_plain);
+                free(argv_plain);
+            } else if (joined < 0) {
+                goto error;
+            }
+
+            psutil_free_cstrings_array(argv, argc);
+        }
+    }
+
+    /* If we can't read process memory or can't decode the result
+     * then return args from /proc. */
+    if (!py_args) {
+        PyErr_Clear();
+        py_args = PyUnicode_DecodeFSDefault(info.pr_psargs);
+    }
+
+    /* Both methods has been failed. */
     if (!py_args)
+        goto error;
+
+    py_retlist = Py_BuildValue("OO", py_name, py_args);
+    if (!py_retlist)
+        goto error;
+
+    Py_DECREF(py_name);
+    Py_DECREF(py_args);
+    return py_retlist;
+
+error:
+    Py_XDECREF(py_name);
+    Py_XDECREF(py_args);
+    Py_XDECREF(py_retlist);
+    return NULL;
+}
+
+
+/*
+ * Return process environ block.
+ */
+static PyObject *
+psutil_proc_environ(PyObject *self, PyObject *args) {
+    int pid;
+    char path[1000];
+    psinfo_t info;
+    const char *procfs_path;
+    char **env = NULL;
+    ssize_t env_count = -1;
+    char *dm;
+    int i = 0;
+    PyObject *py_envname = NULL;
+    PyObject *py_envval = NULL;
+    PyObject *py_retdict = PyDict_New();
+
+    if (! py_retdict)
+        return PyErr_NoMemory();
+
+    if (! PyArg_ParseTuple(args, "is", &pid, &procfs_path))
         return NULL;
-    return Py_BuildValue("OO", py_name, py_args);
-#else
-    return Py_BuildValue("ss", info.pr_fname, info.pr_psargs);
-#endif
+
+    sprintf(path, "%s/%i/psinfo", procfs_path, pid);
+    if (! psutil_file_to_struct(path, (void *)&info, sizeof(info)))
+        goto error;
+
+    if (! info.pr_envp) {
+        AccessDenied("");
+        goto error;
+    }
+
+    env = psutil_read_raw_env(info, procfs_path, &env_count);
+    if (! env && env_count != 0)
+        goto error;
+
+    for (i=0; i<env_count; i++) {
+        if (! env[i])
+            break;
+
+        dm = strchr(env[i], '=');
+        if (! dm)
+            continue;
+
+        *dm = '\0';
+
+        py_envname = PyUnicode_DecodeFSDefault(env[i]);
+        if (! py_envname)
+            goto error;
+
+        py_envval = PyUnicode_DecodeFSDefault(dm+1);
+        if (! py_envname)
+            goto error;
+
+        if (PyDict_SetItem(py_retdict, py_envname, py_envval) < 0)
+            goto error;
+
+        Py_DECREF(py_envname);
+        Py_DECREF(py_envval);
+    }
+
+    psutil_free_cstrings_array(env, env_count);
+    return py_retdict;
+
+ error:
+    if (env && env_count >= 0)
+        psutil_free_cstrings_array(env, env_count);
+
+    Py_XDECREF(py_envname);
+    Py_XDECREF(py_envval);
+    Py_XDECREF(py_retdict);
+    return NULL;
 }
 
 
@@ -177,13 +353,11 @@ psutil_proc_cpu_num(PyObject *self, PyObject *args) {
     int pid;
     char path[1000];
     struct prheader header;
-    struct lwpsinfo *lwp;
-    char *lpsinfo = NULL;
-    char *ptr = NULL;
+    struct lwpsinfo *lwp = NULL;
     int nent;
     int size;
     int proc_num;
-    size_t nbytes;
+    ssize_t nbytes;
     const char *procfs_path;
 
     if (! PyArg_ParseTuple(args, "is", &pid, &procfs_path))
@@ -211,14 +385,14 @@ psutil_proc_cpu_num(PyObject *self, PyObject *args) {
     // malloc
     nent = header.pr_nent;
     size = header.pr_entsize * nent;
-    ptr = lpsinfo = malloc(size);
-    if (lpsinfo == NULL) {
+    lwp = malloc(size);
+    if (lwp == NULL) {
         PyErr_NoMemory();
         goto error;
     }
 
     // read the rest
-    nbytes = pread(fd, lpsinfo, size, sizeof(header));
+    nbytes = pread(fd, lwp, size, sizeof(header));
     if (nbytes == -1) {
         PyErr_SetFromErrno(PyExc_OSError);
         goto error;
@@ -230,20 +404,15 @@ psutil_proc_cpu_num(PyObject *self, PyObject *args) {
     }
 
     // done
-    lwp = (lwpsinfo_t *)ptr;
     proc_num = lwp->pr_onpro;
     close(fd);
-    free(ptr);
-    free(lpsinfo);
+    free(lwp);
     return Py_BuildValue("i", proc_num);
 
 error:
-    if (fd != NULL)
+    if (fd != -1)
         close(fd);
-    if (ptr != NULL)
-        free(ptr);
-    if (lpsinfo != NULL)
-        free(lpsinfo);
+    free(lwp);
     return NULL;
 }
 
@@ -270,7 +439,7 @@ psutil_proc_cred(PyObject *self, PyObject *args) {
 
 
 /*
- * Return process uids/gids as a Python tuple.
+ * Return process voluntary and involuntary context switches as a Python tuple.
  */
 static PyObject *
 psutil_proc_num_ctx_switches(PyObject *self, PyObject *args) {
@@ -296,7 +465,7 @@ psutil_proc_num_ctx_switches(PyObject *self, PyObject *args) {
  * - 'pr_ioch' is a sum of chars read and written, with no distinction
  * - 'pr_inblk' and 'pr_oublk', which should be the number of bytes
  *    read and written, hardly increase and according to:
- *    http://www.brendangregg.com/Perf/paper_diskubyp1.pdf
+ *    http://www.brendangregg.com/Solaris/paper_diskubyp1.pdf
  *    ...they should be meaningless anyway.
  *
 static PyObject*
@@ -316,7 +485,7 @@ proc_io_counters(PyObject* self, PyObject* args) {
     // *and* written.
     // 'pr_inblk' and 'pr_oublk' should be expressed in blocks of
     // 8KB according to:
-    // http://www.brendangregg.com/Perf/paper_diskubyp1.pdf  (pag. 8)
+    // http://www.brendangregg.com/Solaris/paper_diskubyp1.pdf (pag. 8)
     return Py_BuildValue("kkkk",
                          info.pr_ioch,
                          info.pr_ioch,
@@ -448,40 +617,60 @@ psutil_swap_mem(PyObject *self, PyObject *args) {
 static PyObject *
 psutil_users(PyObject *self, PyObject *args) {
     struct utmpx *ut;
-    PyObject *py_retlist = PyList_New(0);
     PyObject *py_tuple = NULL;
+    PyObject *py_username = NULL;
+    PyObject *py_tty = NULL;
+    PyObject *py_hostname = NULL;
     PyObject *py_user_proc = NULL;
+    PyObject *py_retlist = PyList_New(0);
 
     if (py_retlist == NULL)
         return NULL;
 
+    setutxent();
     while (NULL != (ut = getutxent())) {
         if (ut->ut_type == USER_PROCESS)
             py_user_proc = Py_True;
         else
             py_user_proc = Py_False;
+        py_username = PyUnicode_DecodeFSDefault(ut->ut_user);
+        if (! py_username)
+            goto error;
+        py_tty = PyUnicode_DecodeFSDefault(ut->ut_line);
+        if (! py_tty)
+            goto error;
+        py_hostname = PyUnicode_DecodeFSDefault(ut->ut_host);
+        if (! py_hostname)
+            goto error;
         py_tuple = Py_BuildValue(
-            "(sssfO)",
-            ut->ut_user,              // username
-            ut->ut_line,              // tty
-            ut->ut_host,              // hostname
+            "(OOOfOi)",
+            py_username,              // username
+            py_tty,                   // tty
+            py_hostname,              // hostname
             (float)ut->ut_tv.tv_sec,  // tstamp
-            py_user_proc);            // (bool) user process
+            py_user_proc,             // (bool) user process
+            ut->ut_pid                // process id
+        );
         if (py_tuple == NULL)
             goto error;
         if (PyList_Append(py_retlist, py_tuple))
             goto error;
+        Py_DECREF(py_username);
+        Py_DECREF(py_tty);
+        Py_DECREF(py_hostname);
         Py_DECREF(py_tuple);
     }
-    endutent();
+    endutxent();
 
     return py_retlist;
 
 error:
+    Py_XDECREF(py_username);
+    Py_XDECREF(py_tty);
+    Py_XDECREF(py_hostname);
     Py_XDECREF(py_tuple);
     Py_DECREF(py_retlist);
-    if (ut != NULL)
-        endutent();
+    endutxent();
     return NULL;
 }
 
@@ -494,8 +683,10 @@ static PyObject *
 psutil_disk_partitions(PyObject *self, PyObject *args) {
     FILE *file;
     struct mnttab mt;
-    PyObject *py_retlist = PyList_New(0);
+    PyObject *py_dev = NULL;
+    PyObject *py_mountp = NULL;
     PyObject *py_tuple = NULL;
+    PyObject *py_retlist = PyList_New(0);
 
     if (py_retlist == NULL)
         return NULL;
@@ -507,23 +698,32 @@ psutil_disk_partitions(PyObject *self, PyObject *args) {
     }
 
     while (getmntent(file, &mt) == 0) {
+        py_dev = PyUnicode_DecodeFSDefault(mt.mnt_special);
+        if (! py_dev)
+            goto error;
+        py_mountp = PyUnicode_DecodeFSDefault(mt.mnt_mountp);
+        if (! py_mountp)
+            goto error;
         py_tuple = Py_BuildValue(
-            "(ssss)",
-            mt.mnt_special,   // device
-            mt.mnt_mountp,    // mount point
+            "(OOss)",
+            py_dev,           // device
+            py_mountp,        // mount point
             mt.mnt_fstype,    // fs type
             mt.mnt_mntopts);  // options
         if (py_tuple == NULL)
             goto error;
         if (PyList_Append(py_retlist, py_tuple))
             goto error;
+        Py_DECREF(py_dev);
+        Py_DECREF(py_mountp);
         Py_DECREF(py_tuple);
-
     }
     fclose(file);
     return py_retlist;
 
 error:
+    Py_XDECREF(py_dev);
+    Py_XDECREF(py_mountp);
     Py_XDECREF(py_tuple);
     Py_DECREF(py_retlist);
     if (file != NULL)
@@ -651,7 +851,7 @@ psutil_proc_memory_maps(PyObject *self, PyObject *args) {
     int fd = -1;
     char path[1000];
     char perms[10];
-    char *name;
+    const char *name;
     struct stat st;
     pstatus_t status;
 
@@ -664,6 +864,7 @@ psutil_proc_memory_maps(PyObject *self, PyObject *args) {
     const char *procfs_path;
 
     PyObject *py_tuple = NULL;
+    PyObject *py_path = NULL;
     PyObject *py_retlist = PyList_New(0);
 
     if (py_retlist == NULL)
@@ -710,7 +911,7 @@ psutil_proc_memory_maps(PyObject *self, PyObject *args) {
         pr_addr_sz = p->pr_vaddr + p->pr_size;
 
         // perms
-        sprintf(perms, "%c%c%c%c%c%c", p->pr_mflags & MA_READ ? 'r' : '-',
+        sprintf(perms, "%c%c%c%c", p->pr_mflags & MA_READ ? 'r' : '-',
                 p->pr_mflags & MA_WRITE ? 'w' : '-',
                 p->pr_mflags & MA_EXEC ? 'x' : '-',
                 p->pr_mflags & MA_SHARED ? 's' : '-');
@@ -742,19 +943,23 @@ psutil_proc_memory_maps(PyObject *self, PyObject *args) {
             }
         }
 
+        py_path = PyUnicode_DecodeFSDefault(name);
+        if (! py_path)
+            goto error;
         py_tuple = Py_BuildValue(
-            "iisslll",
-            p->pr_vaddr,
-            pr_addr_sz,
+            "kksOkkk",
+            (unsigned long)p->pr_vaddr,
+            (unsigned long)pr_addr_sz,
             perms,
-            name,
-            (long)p->pr_rss * p->pr_pagesize,
-            (long)p->pr_anon * p->pr_pagesize,
-            (long)p->pr_locked * p->pr_pagesize);
+            py_path,
+            (unsigned long)p->pr_rss * p->pr_pagesize,
+            (unsigned long)p->pr_anon * p->pr_pagesize,
+            (unsigned long)p->pr_locked * p->pr_pagesize);
         if (!py_tuple)
             goto error;
         if (PyList_Append(py_retlist, py_tuple))
             goto error;
+        Py_DECREF(py_path);
         Py_DECREF(py_tuple);
 
         // increment pointer
@@ -769,6 +974,7 @@ error:
     if (fd != -1)
         close(fd);
     Py_XDECREF(py_tuple);
+    Py_XDECREF(py_path);
     Py_DECREF(py_retlist);
     if (xmap != NULL)
         free(xmap);
@@ -907,23 +1113,23 @@ static PyObject *
 psutil_net_connections(PyObject *self, PyObject *args) {
     long pid;
     int sd = 0;
-    mib2_tcpConnEntry_t *tp = NULL;
-    mib2_udpEntry_t     *ude;
+    mib2_tcpConnEntry_t tp;
+    mib2_udpEntry_t     ude;
 #if defined(AF_INET6)
-    mib2_tcp6ConnEntry_t *tp6;
-    mib2_udp6Entry_t     *ude6;
+    mib2_tcp6ConnEntry_t tp6;
+    mib2_udp6Entry_t     ude6;
 #endif
     char buf[512];
     int i, flags, getcode, num_ent, state;
-    char lip[200], rip[200];
+    char lip[INET6_ADDRSTRLEN], rip[INET6_ADDRSTRLEN];
     int lport, rport;
     int processed_pid;
     int databuf_init = 0;
     struct strbuf ctlbuf, databuf;
-    struct T_optmgmt_req *tor = (struct T_optmgmt_req *)buf;
-    struct T_optmgmt_ack *toa = (struct T_optmgmt_ack *)buf;
-    struct T_error_ack   *tea = (struct T_error_ack *)buf;
-    struct opthdr        *mibhdr;
+    struct T_optmgmt_req tor = {0};
+    struct T_optmgmt_ack toa = {0};
+    struct T_error_ack   tea = {0};
+    struct opthdr        mibhdr = {0};
 
     PyObject *py_retlist = PyList_New(0);
     PyObject *py_tuple = NULL;
@@ -941,9 +1147,7 @@ psutil_net_connections(PyObject *self, PyObject *args) {
         goto error;
     }
 
-    /*
-    XXX - These 2 are used in ifconfig.c but they seem unnecessary
-    ret = ioctl(sd, I_PUSH, "tcp");
+    int ret = ioctl(sd, I_PUSH, "tcp");
     if (ret == -1) {
         PyErr_SetFromErrno(PyExc_OSError);
         goto error;
@@ -953,23 +1157,28 @@ psutil_net_connections(PyObject *self, PyObject *args) {
         PyErr_SetFromErrno(PyExc_OSError);
         goto error;
     }
-    */
-
+    //
     // OK, this mess is basically copied and pasted from nxsensor project
     // which copied and pasted it from netstat source code, mibget()
     // function.  Also see:
     // http://stackoverflow.com/questions/8723598/
-    tor->PRIM_type = T_SVR4_OPTMGMT_REQ;
-    tor->OPT_offset = sizeof (struct T_optmgmt_req);
-    tor->OPT_length = sizeof (struct opthdr);
-    tor->MGMT_flags = T_CURRENT;
-    mibhdr = (struct opthdr *)&tor[1];
-    mibhdr->level = EXPER_IP_AND_ALL_IRES;
-    mibhdr->name  = 0;
-    mibhdr->len   = 0;
+    tor.PRIM_type = T_SVR4_OPTMGMT_REQ;
+    tor.OPT_offset = sizeof (struct T_optmgmt_req);
+    tor.OPT_length = sizeof (struct opthdr);
+    tor.MGMT_flags = T_CURRENT;
+    mibhdr.level = MIB2_IP;
+    mibhdr.name  = 0;
+
+#ifdef NEW_MIB_COMPLIANT
+    mibhdr.len   = 1;
+#else
+    mibhdr.len   = 0;
+#endif
+    memcpy(buf, &tor, sizeof tor);
+    memcpy(buf + tor.OPT_offset, &mibhdr, sizeof mibhdr);
 
     ctlbuf.buf = buf;
-    ctlbuf.len = tor->OPT_offset + tor->OPT_length;
+    ctlbuf.len = tor.OPT_offset + tor.OPT_length;
     flags = 0;  // request to be sent in non-priority
 
     if (putmsg(sd, &ctlbuf, (struct strbuf *)0, flags) == -1) {
@@ -977,38 +1186,41 @@ psutil_net_connections(PyObject *self, PyObject *args) {
         goto error;
     }
 
-    mibhdr = (struct opthdr *)&toa[1];
     ctlbuf.maxlen = sizeof (buf);
-
     for (;;) {
         flags = 0;
         getcode = getmsg(sd, &ctlbuf, (struct strbuf *)0, &flags);
+        memcpy(&toa, buf, sizeof toa);
+        memcpy(&tea, buf, sizeof tea);
 
         if (getcode != MOREDATA ||
-                ctlbuf.len < sizeof (struct T_optmgmt_ack) ||
-                toa->PRIM_type != T_OPTMGMT_ACK ||
-                toa->MGMT_flags != T_SUCCESS)
+                ctlbuf.len < (int)sizeof (struct T_optmgmt_ack) ||
+                toa.PRIM_type != T_OPTMGMT_ACK ||
+                toa.MGMT_flags != T_SUCCESS)
         {
             break;
         }
-        if (ctlbuf.len >= sizeof (struct T_error_ack) &&
-                tea->PRIM_type == T_ERROR_ACK)
+        if (ctlbuf.len >= (int)sizeof (struct T_error_ack) &&
+                tea.PRIM_type == T_ERROR_ACK)
         {
             PyErr_SetString(PyExc_RuntimeError, "ERROR_ACK");
             goto error;
         }
         if (getcode == 0 &&
-                ctlbuf.len >= sizeof (struct T_optmgmt_ack) &&
-                toa->PRIM_type == T_OPTMGMT_ACK &&
-                toa->MGMT_flags == T_SUCCESS)
+                ctlbuf.len >= (int)sizeof (struct T_optmgmt_ack) &&
+                toa.PRIM_type == T_OPTMGMT_ACK &&
+                toa.MGMT_flags == T_SUCCESS)
         {
             PyErr_SetString(PyExc_RuntimeError, "ERROR_T_OPTMGMT_ACK");
             goto error;
         }
 
-        databuf.maxlen = mibhdr->len;
+        memset(&mibhdr, 0x0, sizeof(mibhdr));
+        memcpy(&mibhdr, buf + toa.OPT_offset, toa.OPT_length);
+
+        databuf.maxlen = mibhdr.len;
         databuf.len = 0;
-        databuf.buf = (char *)malloc((int)mibhdr->len);
+        databuf.buf = (char *)malloc((int)mibhdr.len);
         if (!databuf.buf) {
             PyErr_NoMemory();
             goto error;
@@ -1023,18 +1235,22 @@ psutil_net_connections(PyObject *self, PyObject *args) {
         }
 
         // TCPv4
-        if (mibhdr->level == MIB2_TCP && mibhdr->name == MIB2_TCP_13) {
-            tp = (mib2_tcpConnEntry_t *)databuf.buf;
-            num_ent = mibhdr->len / sizeof(mib2_tcpConnEntry_t);
-            for (i = 0; i < num_ent; i++, tp++) {
-                processed_pid = tp->tcpConnCreationProcess;
+        if (mibhdr.level == MIB2_TCP && mibhdr.name == MIB2_TCP_13) {
+            num_ent = mibhdr.len / sizeof(mib2_tcpConnEntry_t);
+            for (i = 0; i < num_ent; i++) {
+                memcpy(&tp, databuf.buf + i * sizeof tp, sizeof tp);
+#ifdef NEW_MIB_COMPLIANT
+                processed_pid = tp.tcpConnCreationProcess;
+#else
+                processed_pid = 0;
+#endif
                 if (pid != -1 && processed_pid != pid)
                     continue;
                 // construct local/remote addresses
-                inet_ntop(AF_INET, &tp->tcpConnLocalAddress, lip, sizeof(lip));
-                inet_ntop(AF_INET, &tp->tcpConnRemAddress, rip, sizeof(rip));
-                lport = tp->tcpConnLocalPort;
-                rport = tp->tcpConnRemPort;
+                inet_ntop(AF_INET, &tp.tcpConnLocalAddress, lip, sizeof(lip));
+                inet_ntop(AF_INET, &tp.tcpConnRemAddress, rip, sizeof(rip));
+                lport = tp.tcpConnLocalPort;
+                rport = tp.tcpConnRemPort;
 
                 // contruct python tuple/list
                 py_laddr = Py_BuildValue("(si)", lip, lport);
@@ -1047,7 +1263,7 @@ psutil_net_connections(PyObject *self, PyObject *args) {
                 }
                 if (!py_raddr)
                     goto error;
-                state = tp->tcpConnEntryInfo.ce_state;
+                state = tp.tcpConnEntryInfo.ce_state;
 
                 // add item
                 py_tuple = Py_BuildValue("(iiiNNiI)", -1, AF_INET, SOCK_STREAM,
@@ -1062,20 +1278,24 @@ psutil_net_connections(PyObject *self, PyObject *args) {
         }
 #if defined(AF_INET6)
         // TCPv6
-        else if (mibhdr->level == MIB2_TCP6 && mibhdr->name == MIB2_TCP6_CONN)
+        else if (mibhdr.level == MIB2_TCP6 && mibhdr.name == MIB2_TCP6_CONN)
         {
-            tp6 = (mib2_tcp6ConnEntry_t *)databuf.buf;
-            num_ent = mibhdr->len / sizeof(mib2_tcp6ConnEntry_t);
+            num_ent = mibhdr.len / sizeof(mib2_tcp6ConnEntry_t);
 
-            for (i = 0; i < num_ent; i++, tp6++) {
-                processed_pid = tp6->tcp6ConnCreationProcess;
+            for (i = 0; i < num_ent; i++) {
+                memcpy(&tp6, databuf.buf + i * sizeof tp6, sizeof tp6);
+#ifdef NEW_MIB_COMPLIANT
+                processed_pid = tp6.tcp6ConnCreationProcess;
+#else
+        		processed_pid = 0;
+#endif
                 if (pid != -1 && processed_pid != pid)
                     continue;
                 // construct local/remote addresses
-                inet_ntop(AF_INET6, &tp6->tcp6ConnLocalAddress, lip, sizeof(lip));
-                inet_ntop(AF_INET6, &tp6->tcp6ConnRemAddress, rip, sizeof(rip));
-                lport = tp6->tcp6ConnLocalPort;
-                rport = tp6->tcp6ConnRemPort;
+                inet_ntop(AF_INET6, &tp6.tcp6ConnLocalAddress, lip, sizeof(lip));
+                inet_ntop(AF_INET6, &tp6.tcp6ConnRemAddress, rip, sizeof(rip));
+                lport = tp6.tcp6ConnLocalPort;
+                rport = tp6.tcp6ConnRemPort;
 
                 // contruct python tuple/list
                 py_laddr = Py_BuildValue("(si)", lip, lport);
@@ -1087,7 +1307,7 @@ psutil_net_connections(PyObject *self, PyObject *args) {
                     py_raddr = Py_BuildValue("()");
                 if (!py_raddr)
                     goto error;
-                state = tp6->tcp6ConnEntryInfo.ce_state;
+                state = tp6.tcp6ConnEntryInfo.ce_state;
 
                 // add item
                 py_tuple = Py_BuildValue("(iiiNNiI)", -1, AF_INET6, SOCK_STREAM,
@@ -1101,11 +1321,16 @@ psutil_net_connections(PyObject *self, PyObject *args) {
         }
 #endif
         // UDPv4
-        else if (mibhdr->level == MIB2_UDP || mibhdr->level == MIB2_UDP_ENTRY) {
-            ude = (mib2_udpEntry_t *)databuf.buf;
-            num_ent = mibhdr->len / sizeof(mib2_udpEntry_t);
-            for (i = 0; i < num_ent; i++, ude++) {
-                processed_pid = ude->udpCreationProcess;
+        else if (mibhdr.level == MIB2_UDP || mibhdr.level == MIB2_UDP_ENTRY) {
+            num_ent = mibhdr.len / sizeof(mib2_udpEntry_t);
+	    assert(num_ent * sizeof(mib2_udpEntry_t) == mibhdr.len);
+            for (i = 0; i < num_ent; i++) {
+                memcpy(&ude, databuf.buf + i * sizeof ude, sizeof ude);
+#ifdef NEW_MIB_COMPLIANT
+                processed_pid = ude.udpCreationProcess;
+#else
+                processed_pid = 0;
+#endif
                 if (pid != -1 && processed_pid != pid)
                     continue;
                 // XXX Very ugly hack! It seems we get here only the first
@@ -1115,8 +1340,8 @@ psutil_net_connections(PyObject *self, PyObject *args) {
                 // to do other than skipping.
                 if (processed_pid > 131072)
                     continue;
-                inet_ntop(AF_INET, &ude->udpLocalAddress, lip, sizeof(lip));
-                lport = ude->udpLocalPort;
+                inet_ntop(AF_INET, &ude.udpLocalAddress, lip, sizeof(lip));
+                lport = ude.udpLocalPort;
                 py_laddr = Py_BuildValue("(si)", lip, lport);
                 if (!py_laddr)
                     goto error;
@@ -1135,17 +1360,21 @@ psutil_net_connections(PyObject *self, PyObject *args) {
         }
 #if defined(AF_INET6)
         // UDPv6
-        else if (mibhdr->level == MIB2_UDP6 ||
-                    mibhdr->level == MIB2_UDP6_ENTRY)
+        else if (mibhdr.level == MIB2_UDP6 ||
+                    mibhdr.level == MIB2_UDP6_ENTRY)
             {
-            ude6 = (mib2_udp6Entry_t *)databuf.buf;
-            num_ent = mibhdr->len / sizeof(mib2_udp6Entry_t);
-            for (i = 0; i < num_ent; i++, ude6++) {
-                processed_pid = ude6->udp6CreationProcess;
+            num_ent = mibhdr.len / sizeof(mib2_udp6Entry_t);
+            for (i = 0; i < num_ent; i++) {
+                memcpy(&ude6, databuf.buf + i * sizeof ude6, sizeof ude6);
+#ifdef NEW_MIB_COMPLIANT
+                processed_pid = ude6.udp6CreationProcess;
+#else
+                processed_pid = 0;
+#endif
                 if (pid != -1 && processed_pid != pid)
                     continue;
-                inet_ntop(AF_INET6, &ude6->udp6LocalAddress, lip, sizeof(lip));
-                lport = ude6->udp6LocalPort;
+                inet_ntop(AF_INET6, &ude6.udp6LocalAddress, lip, sizeof(lip));
+                lport = ude6.udp6LocalPort;
                 py_laddr = Py_BuildValue("(si)", lip, lport);
                 if (!py_laddr)
                     goto error;
@@ -1187,20 +1416,20 @@ psutil_boot_time(PyObject *self, PyObject *args) {
     float boot_time = 0.0;
     struct utmpx *ut;
 
+    setutxent();
     while (NULL != (ut = getutxent())) {
         if (ut->ut_type == BOOT_TIME) {
             boot_time = (float)ut->ut_tv.tv_sec;
             break;
         }
     }
-    endutent();
-    if (boot_time != 0.0) {
-        return Py_BuildValue("f", boot_time);
-    }
-    else {
+    endutxent();
+    if (fabs(boot_time) < 0.000001) {
+        /* could not find BOOT_TIME in getutxent loop */
         PyErr_SetString(PyExc_RuntimeError, "can't determine boot time");
         return NULL;
     }
+    return Py_BuildValue("f", boot_time);
 }
 
 
@@ -1402,12 +1631,13 @@ error:
  */
 static PyMethodDef
 PsutilMethods[] = {
-
     // --- process-related functions
     {"proc_basic_info", psutil_proc_basic_info, METH_VARARGS,
      "Return process ppid, rss, vms, ctime, nice, nthreads, status and tty"},
     {"proc_name_and_args", psutil_proc_name_and_args, METH_VARARGS,
      "Return process name and args."},
+    {"proc_environ", psutil_proc_environ, METH_VARARGS,
+      "Return process environment."},
     {"proc_cpu_times", psutil_proc_cpu_times, METH_VARARGS,
      "Return process user and system CPU times."},
     {"proc_cred", psutil_proc_cred, METH_VARARGS,
@@ -1445,6 +1675,10 @@ PsutilMethods[] = {
     {"cpu_stats", psutil_cpu_stats, METH_VARARGS,
      "Return CPU statistics"},
 
+    // --- others
+    {"set_testing", psutil_set_testing, METH_NOARGS,
+     "Set psutil in testing mode"},
+
     {NULL, NULL, 0, NULL}
 };
 
@@ -1500,6 +1734,12 @@ void init_psutil_sunos(void)
 #else
     PyObject *module = Py_InitModule("_psutil_sunos", PsutilMethods);
 #endif
+    if (module == NULL)
+        INITERROR;
+
+    if (psutil_setup() != 0)
+        INITERROR;
+
     PyModule_AddIntConstant(module, "version", PSUTIL_VERSION);
 
     PyModule_AddIntConstant(module, "SSLEEP", SSLEEP);
